<?php

// session handling package
// wraps standard PHP session handling (which is sort of crude)
// - ckl --
// Unserialize added from TSz :-)

require_once 'XML/Serializer.php'; 
require_once 'XML/Unserializer.php';

class XmlSerializableObject {
    var $encoding;
    // a wrapper class to access the pear xml serializer
    
    // serialization options
    private static $opt = array(
        'indent' => '    ',
        'typeHints' => TRUE, 
        );
    
    private function setupOpts(&$opts) {
        $opt = self::$opt;
        if ($opts != null)  {
            if (!is_array($opts)) $opts = array('rootName' => $opts);
            foreach ($opts as $o => $v) {
                $opt[$o] = $v;
            }
        }
        $opt['addDecl'] = true;
        $opt['encoding'] = $this->encoding;
        return $opt;
    }
    
    public function serializeObj(&$obj, $opts = null) {
        // return xml representation of object, may be used on derived objects or on just any var/object
        // normally, you would use the serialize() interface below
        // 'opts' can be either the name of the object or an array of serializer options which are overlayed to the default options
        $s = new XML_Serializer($this->setupOpts($opts)); 
        if (!PEAR::isError($s->serialize($obj))) {
            return $s->getSerializedData();
        } else {
            return "<error/>";
        }
    }
    
    public function serialize($name = null) {
        // wraps serializeObj when used on the object itself
        return self::serializeObj($this, $name);
    }
    
    public function unserializeObj(&$file, $opts = null, $isfile = true) {
        $u = new XML_Unserializer($this->setupOpts($opts));
        if (!PEAR::isError($u->unserialize($file, $isfile))) {
            return $u->getUnserializedData();
        } else {
            return null;
        }
    }
    
    public function showHtml($xml) {
        // return html code from xml (in fact, from just any string), just for convenience, does not belong here really
        return "<pre>" . htmlspecialchars($xml) . "</pre>"; 
    }
}

class _mySession extends XmlSerializableObject {
    
    // the real session data, do NOT use this class (private to class mySession)
    
    public $created = false;
    public $begin = 0;
    public $duration = 0;
    public $end = 0;
    public $fresh = false;
    public $id = "";
    
    function __construct ($duration, $scope, $encoding='ISO-8859-1') {
        $this->encoding = $encoding;
        
        // creates a session and sets some session parameters
        session_name($scope);
        
        // ini_set is not the way to solve the issue, as each other php script with sessions will override this value with the default one
        // so the garbage collector will collect data again based on the ini value
        // another possibility would be to set an own session_save_path and to do garbage collection on this path with an own side by side task
        /*** conclusion: set this value fix in the php.ini for the whole server... ***/
        /*if($duration != 0) {
            ini_set("session.gc_maxlifetime", $duration);   // this sets the session life time on the server
        }*/
        //session_cache_expire($duration/60);             // this sets HTTP cache expiration time, won't be usefull here
        session_start();
        
        if (($this->id = session_id()) != "") {
            $this->created = true;
            if (!isset($_SESSION['__session']) || $_SESSION['__session'] == "") {
                // new session, initialize
                $this->duration = $duration;
                $this->fresh = true;
                $this->begin = time();
                if ($this->duration != 0) $this->end = $this->begin + $this->duration;
            } else {
                $this->fresh = false;
            }
        } else {
            $this->created = false;
        }
    }
    
    function __toString() {
        // used when the object is converted into a string (implicitly by php)
        return $this->serialize();
    }
}

class mySession extends XmlSerializableObject {
    //
    // wraps session handling functions
    //
    // note that this class MUST be instantiated BEFORE any html output has been done
    // otherwise, the session will NOT work
    //
    // this will be a cookie based session
    
    public static $s = null; 
    
    // read only access to session properties
    public function created() {
        return self::$s->created;
    }
    public function begin() {
        return self::$s->begin;
    }
    public function duration() {
        return self::$s->duration;
    }
    public function end() {
        return self::$s->end;
    }
    public function fresh() {
        return self::$s->fresh;
    }
    public function id() {
        return self::$s->id;
    }
    public function s() {
        return clone self::$s;
    }
    
    function __construct($duration = 0, $scope = "innovaphone_com", $encoding='ISO-8859-1') {
        $this->encoding = $encoding;
        // print "-> mySession()<br>";
        if (self::$s != null) return;    // multiple instantiations share the same session
        self::$s = new _mySession($duration, $scope, $encoding);
        if (self::$s->fresh) {
            // fresh session, save in session environment
            $_SESSION['__session'] = self::$s;
        } else {
            // old session, restore from session
            self::$s = $_SESSION['__session'];
            self::$s->fresh = false;
        }
    }
    
    function __toString() {
        $r = "<PHPSession>\n";
        foreach ($_SESSION as $i => $j) $r .= self::serializeObj($j, $i) . "\n";
        $r .= "</PHPSession>";
        return $r;
    }
    
    public function save(&$o, $as) {
        // save an object into the php session memory
        $_SESSION[$as] = $o;
    }
    
    public function present($as) {
        // see if an item is in session
        return isset($_SESSION[$as]);
    }
    
    public function value($as) {
        return $_SESSION[$as];
    }
    
    public function restore(&$o, $from) {
        // restore object from name $from into $o
        if (!isset($_SESSION[$from])) {
            die("mySession::restore: '$from' not in session");
        }
        $o = $_SESSION[$from];
    }
    
    public function remove($as) {
        // remove an item previously saved with save from session
        unset($_SESSION[$as]);
    }
    
    public function destroy() {
        // destroy the entire session data
        $_SESSION = array();
        if (isset($_COOKIE[session_name()])) {
            setcookie(session_name(), '', time()-42000, '/');
        }
        session_destroy();
        session_start();
    }
    
};

